<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \Lsc\Wp\Context\Context;
use \Lsc\Wp\Context\ContextOption;
use \Lsc\Wp\Context\RootPanelContextOption;
use \Lsc\Wp\Logger;
use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\ControlPanel;
use \Lsc\Wp\PluginVersion;
use \Lsc\Wp\WPDashMsgs;
use \Lsc\Wp\WPInstallStorage;
use \LsPanel\DALiteSpeedController;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\DAPluginException;
use \LsPanel\DAPluginLogger;
use \LsPanel\DAPluginEscalatedLogger;
use \LsPanel\Extend\DAPanelLsadmContextOption;

/**
 * @since 2.0
 */
class DAEscalate
{

    /**
     * @since 2.0
     * @var int
     */
    const EXIT_SUCC = 0;

    /**
     * @since 2.0
     * @var int
     */
    const EXIT_ERROR = 1;

    /**
     * @since 2.0
     * @var string
     */
    const LSWS_CMD_GET_LICENSE_STATUS = 'GET_LICENSE_STATUS';

    /**
     * @since 2.0
     * @var string
     */
    const LSWS_CMD_GET_MAINMENU_INFO = 'GET_MAINMENU_INFO';

    /**
     * @since 2.0
     * @var string
     */
    const LSWS_CMD_GET_SERIAL_NO = 'GET_SERIAL_NO';

    /**
     * @since 2.0
     * @var string
     */
    const LSWS_CMD_RESTART_LSWS = 'RESTART_LSWS';

    /**
     * @since 2.0
     * @var string
     */
    const LSWS_CMD_SWITCH_LICENSE = 'SWITCH_LICENSE';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_APPLY_VHCONF_CHANGES = 'applyVhConfChanges';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT = 'createVhConfAndSetCacheRoot';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_DA_VH_CACHE_ROOT_SEARCH = 'daVhCacheRootSearch';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_DO_ACTION = 'doAction';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_DATA_FILE_EXISTS = 'dataFileExists';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_FILE_EXISTS = 'fileExists';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_CACHE_ROOT_SETTING = 'getCacheRootSetting';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_CURR_VER = 'getCurrVer';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_DOCROOT_MAP = 'getDocrootMap';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_DATA_FILE_CONTENT = 'getDataFileContent';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING = 'getLswsCacheRootSetting';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_REAL_PATH = 'getRealPath';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_GET_WP_DASH_MSG_DATA = 'getWpDashMsgData';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_IS_CACHE_ENABLED = 'isCacheEnabled';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_PROCESS_TEMP_ESCALATION_LOG = 'processTempEscalationLog';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_REFRESH_VER_LIST = 'refreshVerList';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_REMOVE_FILE = 'removeFile';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_SET_ACTIVE_VER = 'setActiveVer';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_SET_SERVER_CACHE_ROOT = 'setServerCacheRoot';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_UPDATE_DATA_FILE = 'updateDataFile';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_UPDATE_SHARED_CODE = 'updateSharedCode';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_WRITE_DASH_PLUGIN_MD5 = 'writeDashMd5';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_WRITE_TO_LOG = 'WRITE_TO_LOG';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_WRITE_VH_CACHE_ROOT = 'writeVhCacheRoot';

    /**
     * @since 2.0
     * @var string
     */
    const MGR_CMD_WRITE_WP_DASH_MSGS_DATA = 'writeWpDashMsgsData';

    /**
     * @since 2.0
     * @var string
     */
    const TYPE_CMD_LSWS = 'typeCmdLsws';

    /**
     * @since 2.0
     * @var string
     */
    const TYPE_CMD_MGR = 'typeCmdMgr';

    /**
     * @since 2.0
     * @var string
     */
    private $parentPid;

    /**
     * @since 2.0
     * @var string
     */
    private $action;

    /**
     * @since 2.0
     * @var mixed[]
     */
    private $params = array();

    /**
     *
     * @since 2.0
     */
    private function __construct()
    {
        $this->init();
    }

    /**
     *
     * @since 2.0
     */
    private function init()
    {
        require_once __DIR__ . '/autoloader.php';

        $args = $_SERVER['argv'];

        /**
         * Shift out script name.
         */
        array_shift($args);

        $lswsHome = array_shift($args);
        define('LSWS_HOME', $lswsHome);

        $parentPid = array_shift($args);
        $this->parentPid = $parentPid;

        $this->setActionAndParams($args);
    }

    /**
     *
     * @since 2.0
     *
     * @param string[]  $cmdInput
     */
    private function setActionAndParams( $cmdInput )
    {
        $action = array_shift($cmdInput);

        if ( self::getSupportedCmdType($action) == '' ) {
            echo '[ERROR] Illegal input: ' . implode(' ', $_SERVER['argv']) . "\n";
            exit(self::EXIT_ERROR);
        }

        $this->action = $action;

        if ( !empty($cmdInput) ) {
            $this->params = unserialize(array_shift($cmdInput));
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return \Lsc\Wp\Panel\DirectAdmin
     */
    private function getControlPanelInstance()
    {
        return ControlPanel::getClassInstance('\Lsc\Wp\Panel\DirectAdmin');
    }

    /**
     *
     * @since 2.0
     */
    private function e2_restartLsws()
    {
        exec('/sbin/service lshttpd restart', $output);
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoCurrentSerialNo()
    {
        $serial = 'NO SERIAL FOUND';

        if ( defined('LSWS_HOME') ) {

            $serial_file = LSWS_HOME . '/conf/serial.no';
            $trial_key_file = LSWS_HOME . '/conf/trial.key';

            if ( is_file($serial_file) ) {
                $serial = file_get_contents($serial_file);
            }
            else if ( is_file($trial_key_file) ) {
                $serial = 'TRIAL';
            }
        }

        echo "RESULT serial={$serial}\n";
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoAdminUrl()
    {
        //http://' . $_SERVER['SERVER_ADDR'] . ':' . $admin_port . '/
        $conf = LSWS_HOME . '/admin/conf/admin_config.xml';
        $data = file_get_contents($conf);

        $url = 'http';

        /**
         * Check if secure
         */
        if ( $data != ''
                && preg_match("/<secure>(\d)<\/secure>/", $data, $matches)
                && $matches[1] == 1 ) {

            $url .= 's';
        }

        $host = $_SERVER['SERVER_NAME'];

        $port = "7080";

        if ( $data != ''
                && preg_match("/<address>.*:(\d+)<\/address>/", $data, $matches) ) {

            $port = $matches[1];
        }

        $url .= "://{$host}:{$port}/";

        echo "RESULT adminurl={$url}\n";
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoOpenAdminUrl()
    {
        //http://' . $_SERVER['SERVER_ADDR'] . ':' . $admin_port . '/
        $conf = LSWS_HOME . '/admin/conf/admin_config.conf';
        $data = file_get_contents($conf);

        $url = 'http';

        /**
         * Check if secure
         */

        $secureRegex = '/adminListener {(?:.|\n)*\ssecure\s+(\d)\s/';

        if ( $data != ''
                && preg_match($secureRegex, $data, $matches)
                && $matches[1] == 1 ) {

            $url .= 's';
        }

        $host = $_SERVER['SERVER_NAME'];

        $port = "7080";

        $addressRegex = '/adminListener {(?:.|\n)*\saddress\s+\*:(\d+)\s/';

        if ( $data != '' && preg_match($addressRegex, $data, $matches) ) {
            $port = $matches[1];
        }

        $url .= "://{$host}:{$port}/";

        echo "RESULT adminurl={$url}\n";
    }

    /**
     *
     * @since 2.0
     */
    private function e1_echoMainMenuInfo()
    {
        $servType =
                (isset($this->params['serv_type'])) ? $this->params['serv_type'] : 'ent';

        switch ( $servType ) {

            case 'ent':
                $this->e1_echoCurrentSerialNo();
                $this->e1_echoAdminUrl();
                break;

            case 'open':
                $this->e1_echoOpenAdminUrl();
                break;
        }
    }

    /**
     *
     * @since 2.0
     */
    private function e2_echoCurrentLicenseStatus()
    {
        $serialFile = LSWS_HOME . '/conf/serial.no';
        $licenseFile = LSWS_HOME . '/conf/license.key';
        $trialKeyFile = LSWS_HOME . '/conf/trial.key';

        $buf = '';
        $return_var = 0;

        if ( file_exists($licenseFile) ) {
            $result = $this->e2_runLicenseCheck();

            $return_var = $result['return_var'];
            $buf .= DALiteSpeedUtil::getOutputString($result['output']);
        }
        else {

            if ( file_exists($serialFile) ) {
                $cmd = LSWS_HOME . '/bin/lshttpd -r 2>&1';
                exec($cmd, $output, $return_var);

                $buf .= DALiteSpeedUtil::getOutputString($output);
            }

            if ( file_exists($licenseFile) || file_exists($trialKeyFile) ) {
                $result = $this->e2_runLicenseCheck();

                $return_var = $result['return_var'];
                $buf .= DALiteSpeedUtil::getOutputString($result['output']);
            }
            else {
                $return_var = -1;
                $buf .= "Failed to find serial.no or trial key file, abort!";
            }
        }

        if ( $buf != '' ) {
            echo "RESULT status={$buf}\n";
        }

        echo "RESULT return_var={$return_var}\n";
    }

    /**
     *
     * @since 2.0
     *
     * @return mixed[][]
     */
    private function e2_runLicenseCheck()
    {
        $cmd = LSWS_HOME . '/bin/lshttpd -V 2>&1';
        exec($cmd, $output, $return_var);

        return array( 'return_var' => $return_var, 'output' => $output );
    }

    /**
     *
     * @since 2.0
     *
     * @return null
     */
    private function e2_switchLicense()
    {
        $error = '';

        $serial_no =
                (isset($this->params['serial_no'])) ? $this->params['serial_no'] : '';

        $pid = getmypid();

        if ( $serial_no == 'TRIAL' ) {
            $trial_key = LSWS_HOME . '/conf/trial.key';

            if ( !file_exists($trial_key) ) {
                $error = 'Failed to find trial.key file. Aborted. <br /> Please request a trial license '
                        . 'and save it under lsws/conf.';
            }
        }
        else {
            $error = DALiteSpeedUtil::validate_serial_no($serial_no);
        }

        if ( $error != '' ) {
            $this->handleSwitchLicenseError($error, $pid);
            return;
        }

        $renamed = array();

        $error = $this->e2_attemptSwitchLicense($serial_no, $pid, $renamed);

        if ( $error != '' ) {
            $this->handleSwitchLicenseError($error, $pid, $renamed);
            return;
        }

        if ( DALiteSpeedUtil::getLSPID() != 0 ) {
            $this->e2_restartLsws();
        }

        $this->echoSwitchLicenseResult();
    }

    /**
     *
     * @since 2.0
     *
     * @param string    $serial
     * @param int       $pid
     * @param string[]  $renamed
     * @return string
     */
    private function e2_attemptSwitchLicense( $serial, $pid, &$renamed )
    {
        $serial_file = LSWS_HOME . '/conf/serial.no';
        $license_file = LSWS_HOME . '/conf/license.key';

        if ( file_exists($license_file) ) {

            if ( !DALiteSpeedUtil::pidBackup($license_file, $pid) ) {
                return 'Failed to back up old license.key';
            }

            $renamed[] = $license_file;
        }

        if ( file_exists($serial_file) ) {

            if ( !DALiteSpeedUtil::pidBackup($serial_file, $pid) ) {
                return 'Failed to back up old serial.no';
            }

            $renamed[] = $serial_file;
        }

        if ( $serial == 'TRIAL' ) {
            $cmd = LSWS_HOME . '/bin/lshttpd -t 2>&1';
            exec($cmd, $output, $return_var);

            if ( $return_var != 0 ) {
                return 'Test license failed.<br />'
                        . DALiteSpeedUtil::getOutputString($output);
            }

            $cmd = LSWS_HOME . '/bin/lshttpd -V 2>&1';
            exec($cmd, $output, $return_var);

            if ( $return_var != 0 ) {
                return 'Check license failed.<br />'
                        . DALiteSpeedUtil::getOutputString($output);
            }
        }
        else {
            file_put_contents($serial_file, "{$serial}\n");

            $cmd = LSWS_HOME . '/bin/lshttpd -r 2>&1';
            exec($cmd, $output, $return_var);

            if ( $return_var != 0 ) {
                return 'Check license failed.<br />'
                        . DALiteSpeedUtil::getOutputString($output);
            }
        }

        return '';
    }

    /**
     *
     * @since 2.0
     *
     * @param string    $error
     * @param int       $pid
     * @param string[]  $renamed
     */
    private function handleSwitchLicenseError( $error, $pid,
            $renamed = array() )
    {
        foreach ( $renamed as $file ) {
            DALiteSpeedUtil::undoPidBackup($pid, $file);
        }

        $this->echoSwitchLicenseResult($error);
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $error
     */
    private function echoSwitchLicenseResult( $error = '' )
    {
        if ( $error == '' ) {
            $retVar = 0;
            $statusMsg = "<p><b>Successfully switched to new license.</b></p>";
        }
        else {
            $retVar = -1;
            $statusMsg = "<p><b>Failed to switch current license.</b></p><p>{$error}</p>";
        }

        echo "RESULT status={$statusMsg}\n";
        echo "RESULT return_var={$retVar}\n";
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    private function e2_writeToLog()
    {
        if ( LSWS_HOME != '' ) {
            $logDir = LSWS_HOME . '/logs';
        }

        $tempLogFile = DAPluginLogger::TEMP_LOG_BASE . "_{$this->parentPid}.log";

        if ( file_exists($tempLogFile)
                && isset($logDir)
                && file_exists($logDir) ) {

            /**
             * Check that shared code ContextOption.php has been loaded
             * (or load it here) before using it if possible. (optional)
             */

            $sharedContextOption =
                    LSWS_HOME . '/add-ons/webcachemgr/src/Context/ContextOption.php';

            if ( file_exists($sharedContextOption)
                    && include_once $sharedContextOption ) {

                $logFile = "{$logDir}/" . ContextOption::LOG_FILE_NAME;
            }
            else {
                $logFile = "{$logDir}/webcachemgr.log";
            }

            $content = file_get_contents($tempLogFile);

            return (bool)file_put_contents($logFile, $content,
                    FILE_APPEND | LOCK_EX);
        }

        return false;
    }

    /**
     *
     * @since 2.0
     */
    private function e2_setServerCacheRoot()
    {
        $panel = $this->getControlPanelInstance();
        $panel->setServerCacheRoot();
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException
     */
    private function e2_writeVHCacheRoot()
    {
        if ( !isset($this->params['vhConf']) ) {
            throw new LSCMException(
                    'Missing \'vhConf\' arg in escalated e2_writeVHCacheRoot() call.');
        }

        $vhConf = $this->params['vhConf'];

        $panel = $this->getControlPanelInstance();
        $panel->writeVHCacheRoot($vhConf);
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    private function e2_daVhCacheRootSearch()
    {
        $confDir = $this->params['confDir'];

        $panel = $this->getControlPanelInstance();

        return $panel->daVhCacheRootSearch($confDir);
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $confFile
     * @return string
     */
    private function e2_getCacheRootSetting( $confFile = '' )
    {
        if ( $confFile == '' && isset($this->params['confFile']) ) {
            $confFile = $this->params['confFile'];
        }

        $panel = $this->getControlPanelInstance();

        return $panel->getCacheRootSetting($confFile);
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    private function e2_getLSWSCacheRootSetting()
    {
        $panel = $this->getControlPanelInstance();

        return $panel->getLSWSCacheRootSetting();
    }

    /**
     *
     * @since 2.0
     *
     * @return null
     */
    private function e2_createVHConfAndSetCacheRoot()
    {
        $vhConf = (isset($this->params['vhConf'])) ? $this->params['vhConf'] : '';

        if ( $vhConf == '' ) {
            return;
        }

        $panel = $this->getControlPanelInstance();

        $panel->createVHConfAndSetCacheRoot($vhConf);
    }

    /**
     *
     * @since 2.0
     */
    private function e2_applyVHConfChanges()
    {
        $panel = $this->getControlPanelInstance();

        $panel->applyVHConfChanges();
    }

    /**
     *
     * @since 2.0
     *
     * @return mixed[][]
     */
    private function e2_getDocrootMap()
    {
        $panelEnv = $this->getControlPanelInstance();

        return $panelEnv->getDocrootMap();
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    private function e2_isCacheEnabled()
    {
        $panel = $this->getControlPanelInstance();

        return $panel->isCacheEnabled();
    }

    /**
     *
     * @since 2.0
     *
     * @return mixed[][]
     */
    private function e2_doAction()
    {
        $action = (isset($this->params['action'])) ? $this->params['action'] : '';
        $list = (isset($this->params['list'])) ? $this->params['list'] : array();
        $extraArgs =
                (isset($this->params['extraArgs'])) ? $this->params['extraArgs'] : array();

        /**
         * Init DirectAdmin control panel object.
         */
        $this->getControlPanelInstance();

        $dataFileInfo = Context::getLSCMDataFiles();
        $wpInstallStorage = new WPInstallStorage($dataFileInfo['dataFile'],
            $dataFileInfo['custDataFile']);

        $finishedList = $wpInstallStorage->doAction($action, $list, $extraArgs);

        $ret = array (
            'finishedList' => $finishedList,
            'workingQueue' => $wpInstallStorage->getWorkingQueue(),
            'invalidPathErrs' => Logger::getUiMsgs(Logger::UI_ERR)
        );

        return $ret;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    private function e2_getCurrentVersion()
    {
        $currVer = PluginVersion::getCurrentVersion();

        return $currVer;
    }

    /**
     *
     * @since 2.0
     */
    private function e2_setActiveVersion()
    {
        $version = (isset($this->params['ver'])) ? $this->params['ver'] : '';
        $init = (isset($this->params['init'])) ? $this->params['init'] : false;

        $vermgr = PluginVersion::getInstance();
        $vermgr->setActiveVersion($version, $init);
    }

    /**
     *
     * @since 2.0
     */
    private function e2_refreshVersionList()
    {
        /**
         * Trigger refresh version list during init().
         */
        PluginVersion::getInstance();
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    private function e2_fileExists()
    {
        $filePath = (isset($this->params['filePath'])) ? $this->params['filePath'] : '';

        $approvedFilePaths = array(
            '/etc/httpd/conf/extra/httpd-includes.conf',
        );

        if ( !in_array($filePath, $approvedFilePaths) ) {
            return false;
        }

        return file_exists($filePath);
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    private function e2_dataFileExists()
    {
        $filePath = (isset($this->params['filePath'])) ? $this->params['filePath'] : '';

        /**
         * Init DirectAdmin control panel object.
         */
        $this->getControlPanelInstance();

        $dataFileInfo = Context::getLSCMDataFiles();


        $approvedFilePaths = array(
            $dataFileInfo['dataFile'],
            $dataFileInfo['custDataFile']
        );

        if ( !in_array($filePath, $approvedFilePaths) ) {
            return false;
        }

        return file_exists($filePath);
    }

    /**
     * Attempts to get the file contents of a data file based on a pre-approved
     * list.
     *
     * @since 2.0
     *
     * @return string
     */
    private function e2_getDataFileContent()
    {
        $filePath = (isset($this->params['filePath'])) ? $this->params['filePath'] : '';

        $dataFilePaths = Context::getLSCMDataFiles();

        $approvedFilePaths = array(
            $dataFilePaths['dataFile'],
            $dataFilePaths['custDataFile']
        );

        if ( ! in_array($filePath, $approvedFilePaths)
                || ! file_exists($filePath) ) {

            return '';
        }

        return file_get_contents($filePath);
    }

    /**
     * Works similar to PHP's realpath() with the exception of returning false
     * when supplied with an empty string as a path.
     *
     * @since 2.0
     *
     * @return boolean|string
     */
    private function e2_realPath()
    {
        if ( !isset($this->params['filePath']) || $this->params['filePath'] == '' ) {
            return false;
        }

        $filePath = $this->params['filePath'];

        return realpath($filePath);
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     * @throws DAPluginException
     */
    private function e2_removeFile()
    {
        if ( !isset($this->params['id']) ) {
            throw new DAPluginException('Missing \'id\' arg in escalated e2_removeFile() call.');
        }

        $id = $this->params['id'];

        $files = array ( 'dashPlugin' => '/usr/src/litespeed-wp-plugin/dash-notifier' );

        if ( isset($files[$id]) ) {
            $file = $files[$id];

            if ( file_exists($file) ) {

                if ( is_dir($file) ) {
                    return DALiteSpeedUtil::e2_rrmdir($file);
                }
                else {
                    return unlink($file);
                }
            }
        }

        return false;
    }

    /**
     *
     * @since 2.0
     *
     * @return int|false
     * @throws DAPluginException
     */
    private function e2_writeWpDashMsgsData()
    {
        if ( !isset($this->params['msgsJson']) ) {
            throw new DAPluginException(
                    'Missing \'msgsJson\' arg in escalated e2_writeWpDashMsgsData() call.');
        }

        $msgsJson = base64_decode($this->params['msgsJson']);
        $dataFile = LSWS_HOME . '/admin/lscdata/wpDashMsgs.data';

        return file_put_contents($dataFile, $msgsJson);
    }

    /**
     *
     * @since 2.0
     *
     * @return int|false
     * @throws DAPluginException
     */
    private function e2_writeDashPluginMd5()
    {
        if ( !isset($this->params['dashMd5']) ) {
            throw new DAPluginException(
                    'Missing \'dashMd5\' arg in escalated e2_writeDashMd5() call.');
        }

        $dashMd5 = $this->params['dashMd5'];
        $dashMd5File = "/usr/src/litespeed-wp-plugin/dash_md5";

        return file_put_contents($dashMd5File, $dashMd5);
    }

    /**
     *
     * @since 2.0
     *
     * @return sting[]|string[][]
     */
    private function e2_getWpDashMsgData()
    {
        $wpDashMsgs = new WPDashMsgs();
        return $wpDashMsgs->getMsgData();
    }

    /**
     *
     * Get temp escalation log file content and write it to the general
     * plugin temp log. This is used to keep logged messages in the correct
     * order chronologically.
     *
     * @since 2.0
     *
     * @return boolean
     */
    private function e2_processTempEscalationLog()
    {
        $tempEscalationLog = DAPluginEscalatedLogger::getLogFilePath();

        if ( file_exists($tempEscalationLog) ) {
            $tempLog = DAPluginLogger::TEMP_LOG_BASE . "_{$this->parentPid}.log";

            $content = file_get_contents($tempEscalationLog);

            $ret = (bool)file_put_contents($tempLog, $content,
                    FILE_APPEND | LOCK_EX);

            if ( $ret ) {
                unlink($tempEscalationLog);
            }

            return $ret;
        }

        return false;
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    private function e2_updateSharedCode()
    {
        $updateCmd = LSWS_HOME . '/admin/misc/lscmctl --update-lib';

        DALiteSpeedUtil::exec_cmd($updateCmd, $output, $ret);

        return ( $ret == 0 ) ? true : false;
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     * @throws LSCMException
     */
    private function e2_updateDataFile()
    {
        if ( !isset($this->params['dataFile']) ) {
            throw new LSCMException(
                    'Missing \'dataFile\' arg in escalated updateDataFile() call.');
        }

        if ( !isset($this->params['dataFileVer']) ) {
            throw new LSCMException(
                    'Missing \'dataFileVer\' arg in escalated updateDataFile() call.');
        }

        $dataFile = $this->params['dataFile'];
        $dataFileVer = $this->params['dataFileVer'];

        return WPInstallStorage::updateDataFile($dataFile, $dataFileVer);
    }

    /**
     *
     * @since 2.0
     *
     * @param boolean  $asRoot
     * @param string   $cmd
     * @param mixed[]  $args
     * @return string
     */
    private static function getEscalateCmd( $asRoot, $cmd, $args = array() )
    {
        $user = ($asRoot) ? 'super' : 'lsadm';
        $pid = getmypid();

        $execCmd = DALiteSpeedController::ADMIN_CGI_DIR . "/{$user}_script "
                . escapeshellarg(LSWS_HOME) . " {$pid} {$cmd}";

        if ( !empty($args) ) {
            $serialized_args = serialize($args);
            $execCmd .= " " . escapeshellarg($serialized_args);
        }

        return $execCmd;
    }

    /**
     *
     * @since 2.0
     *
     * @param boolean  $asRoot
     * @param string   $cmd
     * @param mixed[]  $args
     * @return mixed
     * @throws DAPluginException
     */
    public static function escalate( $asRoot, $cmd, $args = array() )
    {
        if ( ($cmdType = self::getSupportedCmdType($cmd)) == '' ) {
            throw new DAPluginException("Illegal escalation command {$cmd}.");
        }

        $execCmd = self::getEscalateCmd($asRoot, $cmd, $args);

        DALiteSpeedUtil::exec_cmd($execCmd, $output, $return_var);

        $tempEscalatedLog = DAPluginEscalatedLogger::TEMP_ESCALATED_LOG_BASE
                . '_' . getmypid() . '.log';

        if ( file_exists($tempEscalatedLog) ) {
            DAPluginLogger::processBuffer();
            DAPluginLogger::preCreateLogFile();
            self::handleTempEscalationLog();
        }

        DAPluginLogger::debug(
                "Escalation command {$cmd} = {$return_var}\n{$execCmd}");
        DAPluginLogger::debug('output = ' . var_export($output, true));

        switch ($cmdType) {

            case self::TYPE_CMD_LSWS:
                return self::handleLswsCmdOutput($output);

            case self::TYPE_CMD_MGR:
                return self::handleMgrCmdOutput($output);

            //no default case
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string[]  $output
     * @return string[]
     */
    private static function handleLswsCmdOutput( $output )
    {
        $result = array();

        foreach ( $output as $line ) {

            if ( $line != '' && is_string($line) ) {

                if ( preg_match("/^RESULT (\w+)=(.+)$/", $line, $matches) ) {
                    $key = $matches[1];
                    $result[$key] = $matches[2];
                }
                else {
                    $result['MESG'] .= "{$line}<br />";
                }
            }
        }

        return $result;
    }

    /**
     *
     * @since 2.0
     *
     * @param string[]  $output
     * @return mixed
     * @throws LSCMException
     * @throws DAPluginException
     */
    private static function handleMgrCmdOutput( $output )
    {
        $ret = null;
        $err = '';

        foreach ( $output as $line ) {

            if ( ($pos = strpos($line, '[RESULT]')) !== false ) {
                $result = substr($line, $pos + 9);
                $ret = unserialize(base64_decode($result));

                DAPluginLogger::debug(
                        'Decoded [RESULT] = ' . var_export($ret, true));
            }
            elseif ( $pos = strpos($line, '[ERROR]') !== false ) {
                $err .= substr($line, $pos + 7) . "\n";
            }
            elseif ( $line != '' || !empty($err) ) {
                $err .= "{$line}\n";
            }
        }

        if ( !empty($err) ) {

            if ( defined('SHARED_CODE_LOADED') && SHARED_CODE_LOADED ) {
                throw new LSCMException($err);
            }
            else {
                throw new DAPluginException($err);
            }
        }

        return $ret;
    }

    /**
     *
     * @since 2.0
     */
    private static function handleTempEscalationLog()
    {
        $subAction = self::MGR_CMD_PROCESS_TEMP_ESCALATION_LOG;
        $subCmd = self::getEscalateCmd(true, $subAction);

        exec($subCmd, $subOutput, $subReturn_var);

        DAPluginLogger::debug(
                "Issue sub command {$subAction}={$subReturn_var} \n"
                . "{$subCmd}");
        DAPluginLogger::debug('sub output = ' . var_export($subOutput, true));
    }

    /**
     *
     * @since 2.0
     *
     * @param \Exception $e
     */
    private function exitWithCustExceptionError( $e )
    {
        echo "[ERROR] {$e->getMessage()}\n";
        exit(self::EXIT_ERROR);
    }

    /**
     *
     * @since 2.0
     */
    private function loadSharedCode()
    {
        require_once LSWS_HOME . '/add-ons/webcachemgr/autoloader.php';

        $panelName = 'directadmin';

        if ( posix_getuid() == 0 ) {
            $ctxOption = new RootPanelContextOption($panelName);
        }
        else {
            $ctxOption = new DAPanelLsadmContextOption($panelName);
        }

        Context::initialize($ctxOption);

        if ( version_compare(ControlPanel::PANEL_API_VERSION, '1.9', '>=') ) {
            $tempEscalatedLog = DAPluginEscalatedLogger::getLogFilePath();
            Logger::changeLogFileUsed($tempEscalatedLog);
            Logger::setAdditionalTagInfo(
                    "[{$_SERVER['REMOTE_ADDR']}-" . getmypid() . ']');
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return null|mixed
     */
    private function doSharedCodeDependentAction()
    {
        $this->loadSharedCode();

        $ret = null;

        switch ($this->action) {

            case self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH:
                $ret = $this->e2_daVhCacheRootSearch();
                break;

            case self::MGR_CMD_GET_CACHE_ROOT_SETTING:
                $ret = $this->e2_getCacheRootSetting();
                break;

            case self::MGR_CMD_IS_CACHE_ENABLED:
                $ret = $this->e2_isCacheEnabled();
                break;

            case self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT:
                $this->e2_createVHConfAndSetCacheRoot();
                break;

            case self::MGR_CMD_APPLY_VHCONF_CHANGES:
                $this->e2_applyVHConfChanges();
                break;

            case self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING:
                $ret = $this->e2_getLSWSCacheRootSetting();
                break;

            case self::MGR_CMD_SET_SERVER_CACHE_ROOT:
                $this->e2_setServerCacheRoot();
                break;

            case self::MGR_CMD_WRITE_VH_CACHE_ROOT:
                $this->e2_writeVHCacheRoot();
                break;

            case self::MGR_CMD_REFRESH_VER_LIST:
                $this->e2_refreshVersionList();
                break;

            case self::MGR_CMD_GET_CURR_VER:
                $ret = $this->e2_getCurrentVersion();
                break;

            case self::MGR_CMD_SET_ACTIVE_VER:
                $this->e2_setActiveVersion();
                break;

            case self::MGR_CMD_GET_DOCROOT_MAP:
                $ret = $this->e2_getDocrootMap();
                break;

            case self::MGR_CMD_DO_ACTION:
                $ret = $this->e2_doAction();
                break;

            case self::MGR_CMD_DATA_FILE_EXISTS:
                $ret = $this->e2_dataFileExists();
                break;

            case self::MGR_CMD_GET_DATA_FILE_CONTENT:
                $ret = $this->e2_getDataFileContent();
                break;

            case self::MGR_CMD_GET_WP_DASH_MSG_DATA:
                $ret = $this->e2_getWpDashMsgData();
                break;

            case self::MGR_CMD_UPDATE_DATA_FILE:
                $ret = $this->e2_updateDataFile();
                break;
        }

        return $ret;
    }

    /**
     *
     * @since 2.0
     *
     * @return int
     */
    private function runEscalatedCmd()
    {
        DAPluginEscalatedLogger::InitializeWithParentPid($this->parentPid);
        DAPluginEscalatedLogger::setAdditionalTagInfo(
                "[{$_SERVER['REMOTE_ADDR']}-" . getmypid() . ']');

        $ret = null;

        try {
            switch ($this->action) {

                case self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH:
                case self::MGR_CMD_GET_CACHE_ROOT_SETTING:
                case self::MGR_CMD_IS_CACHE_ENABLED:
                case self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT:
                case self::MGR_CMD_APPLY_VHCONF_CHANGES:
                case self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING:
                case self::MGR_CMD_SET_SERVER_CACHE_ROOT:
                case self::MGR_CMD_WRITE_VH_CACHE_ROOT:
                case self::MGR_CMD_REFRESH_VER_LIST:
                case self::MGR_CMD_GET_CURR_VER:
                case self::MGR_CMD_SET_ACTIVE_VER:
                case self::MGR_CMD_GET_DOCROOT_MAP:
                case self::MGR_CMD_DO_ACTION:
                case self::MGR_CMD_DATA_FILE_EXISTS:
                case self::MGR_CMD_GET_WP_DASH_MSG_DATA:
                case self::MGR_CMD_GET_DATA_FILE_CONTENT:
                case self::MGR_CMD_UPDATE_DATA_FILE:
                    $ret = $this->doSharedCodeDependentAction();
                    break;

                case self::MGR_CMD_REMOVE_FILE:
                    $ret = $this->e2_removeFile();
                    break;

                case self::MGR_CMD_WRITE_WP_DASH_MSGS_DATA:
                    $ret = $this->e2_writeWpDashMsgsData();
                    break;

                case self::MGR_CMD_WRITE_DASH_PLUGIN_MD5:
                    $this->e2_writeDashPluginMd5();
                    break;

                case self::LSWS_CMD_GET_LICENSE_STATUS:
                    $this->e2_echoCurrentLicenseStatus();
                    break;

                case self::LSWS_CMD_GET_MAINMENU_INFO:
                    $this->e1_echoMainMenuInfo();
                    break;

                case self::LSWS_CMD_GET_SERIAL_NO:
                    $this->e1_echoCurrentSerialNo();
                    break;

                case self::LSWS_CMD_RESTART_LSWS:
                    $this->e2_restartLsws();
                    break;

                case self::LSWS_CMD_SWITCH_LICENSE:
                    $this->e2_switchLicense();
                    break;

                case self::MGR_CMD_WRITE_TO_LOG:
                    $ret = $this->e2_writeToLog();
                    break;

                case self::MGR_CMD_GET_REAL_PATH:
                    $ret = $this->e2_realPath();
                    break;

                case self::MGR_CMD_PROCESS_TEMP_ESCALATION_LOG:
                    $ret = $this->e2_processTempEscalationLog();
                    break;

                case self::MGR_CMD_UPDATE_SHARED_CODE:
                    $ret = $this->e2_updateSharedCode();
                    break;

                case self::MGR_CMD_FILE_EXISTS:
                    $ret = $this->e2_fileExists();
                    break;

                // no default case
            }

            if ( $ret !== null ) {
                echo '[RESULT] ' . base64_encode(serialize($ret)) . "\n";
            }

            exit(self::EXIT_SUCC);
        }
        catch ( DAPluginException $e ) {
            $this->exitWithCustExceptionError($e);
        }
        catch ( LSCMException $e ) {
            $this->exitWithCustExceptionError($e);
        }
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $cmd
     * @return string Command type or empty string if a matching command is
     *                not found.
     */
    private static function getSupportedCmdType( $cmd )
    {
        $lswsCmds = array(
            self::LSWS_CMD_GET_LICENSE_STATUS,
            self::LSWS_CMD_GET_MAINMENU_INFO,
            self::LSWS_CMD_GET_SERIAL_NO,
            self::LSWS_CMD_RESTART_LSWS,
            self::LSWS_CMD_SWITCH_LICENSE
        );

        if ( in_array($cmd, $lswsCmds) ) {
            return self::TYPE_CMD_LSWS;
        }

        $mgrCmds = array(
            self::MGR_CMD_APPLY_VHCONF_CHANGES,
            self::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT,
            self::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH,
            self::MGR_CMD_DO_ACTION,
            self::MGR_CMD_DATA_FILE_EXISTS,
            self::MGR_CMD_FILE_EXISTS,
            self::MGR_CMD_GET_CACHE_ROOT_SETTING,
            self::MGR_CMD_GET_CURR_VER,
            self::MGR_CMD_GET_DOCROOT_MAP,
            self::MGR_CMD_GET_DATA_FILE_CONTENT,
            self::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING,
            self::MGR_CMD_GET_REAL_PATH,
            self::MGR_CMD_GET_WP_DASH_MSG_DATA,
            self::MGR_CMD_IS_CACHE_ENABLED,
            self::MGR_CMD_PROCESS_TEMP_ESCALATION_LOG,
            self::MGR_CMD_REFRESH_VER_LIST,
            self::MGR_CMD_REMOVE_FILE,
            self::MGR_CMD_SET_ACTIVE_VER,
            self::MGR_CMD_SET_SERVER_CACHE_ROOT,
            self::MGR_CMD_UPDATE_DATA_FILE,
            self::MGR_CMD_UPDATE_SHARED_CODE,
            self::MGR_CMD_WRITE_DASH_PLUGIN_MD5,
            self::MGR_CMD_WRITE_TO_LOG,
            self::MGR_CMD_WRITE_VH_CACHE_ROOT,
            self::MGR_CMD_WRITE_WP_DASH_MSGS_DATA,
        );

        if ( in_array($cmd, $mgrCmds) ) {
            return self::TYPE_CMD_MGR;
        }

        return '';
    }

    /**
     * Check if invoked from shell.
     *
     * @since 2.0
     *
     * @return boolean
     */
    public static function invokedFromShell()
    {
        if ( empty($_SERVER['argv']) ) {
            return false;
        }

        $caller = $_SERVER['argv'][0];

        if ( $caller != '/usr/local/directadmin/plugins/litespeed/admin/DAEscalateBootstrap.php' ) {
            return false;
        }

        return true;
    }

    /**
     * This function should only be executed when this script is included
     * through the DAEscalateBootstrap.php script after escalating with the
     * appropriate escalation script.
     *
     * @since 2.0
     */
    public static function run()
    {
        if ( self::invokedFromShell() ) {
            date_default_timezone_set('UTC');

            $daEscalate = new self();
            $daEscalate->runEscalatedCmd();
        }
    }

}
