<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \Lsc\Wp\Context\Context;
use \Lsc\Wp\Panel\ControlPanel;
use \Lsc\Wp\PluginVersion;
use \Lsc\Wp\View\Model as LscViewModel;
use \LsPanel\DALiteSpeedControllerBase;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\Extend\DAPanelController;
use \LsPanel\Extend\DAPanelWPInstallStorage;
use \LsPanel\View\Model as DAViewModel;

/**
 * @since 2.0
 */
class DAOpenLiteSpeedController extends DALiteSpeedControllerBase
{

    /**
     *
     * @since 2.0
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @since 2.0
     *
     * @return null
     */
    public function run()
    {
        $step = DALiteSpeedUtil::get_request_var('step', 0);
        $mgrStep = DALiteSpeedUtil::get_request_var('mgr_step', 0);
        $dashStep = DALiteSpeedUtil::get_request_var('dash_step', 0);

        $steps = array ( 0, 1, 2, 3);
        $mgrSteps = array( 0, 1, 2, 3, 4);
        $dashSteps = array( 0, 1, 2);

        $do = DALiteSpeedUtil::get_request_var('do');

        $supportedActions = array (
            'restart_lsws',
            'lscwp_manage', 'lscwp_mass_enable_disable', 'lscwpVersionManager',
            'dash_notifier', 'dash_slug_verify',
            'updateSharedCode',
            'main', ''
        );

        if ( !in_array($do, $supportedActions)
                || !in_array($step, $steps)
                || !in_array($mgrStep, $mgrSteps)
                || !in_array($dashStep, $dashSteps)) {

            echo '<h1>Invalid Entrance</h1>';
            return;
        }

        ob_start();

        switch ($do) {

            case 'restart_lsws':
                $viewModel = $this->restartLsws($step);
                break;

            case 'lscwp_manage':
            case 'lscwp_mass_enable_disable':
            case 'lscwpVersionManager':
            case 'dash_notifier':
                $viewModel = $this->doSharedCodeDependentAction($do, $step,
                        $mgrStep, $dashStep);
                break;

            case 'dash_slug_verify':
                $viewModel = $this->dashSlugVerify();
                break;

            case 'updateSharedCode':
                $viewModel = $this->updateSharedCode();
                break;

            case 'main':
            case '':
                $viewModel = $this->getMainMenuViewModel();
                break;

            //no default
        }

        if ( !empty($viewModel) ) {
            $this->display($viewModel);
        }

        if ( defined('LSWS_AJAX_CALL') && LSWS_AJAX_CALL != false ) {
            $this->handleDefaultAjaxResponse(ob_get_clean());
        }

        ob_flush();
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $do
     * @param int     $step
     * @param int     $mgrStep
     * @param int     $dashStep
     * @return object  ViewModel object.
     */
    private function doSharedCodeDependentAction($do, $step, $mgrStep,
            $dashStep)
    {
        if ( ! SHARED_CODE_LOADED ) {
            return (new DAViewModel\SharedCodeUpdateRequiredViewModel());
        }

        if ( version_compare(ControlPanel::PANEL_API_VERSION, '1.9', '<') ) {
            return (new DAViewModel\SharedCodeUpdateRequiredViewModel());
        }

        $panelAPICompatStatus =
                ControlPanel::checkPanelAPICompatibility(self::SHARED_API_VER);

        switch ( $panelAPICompatStatus ) {

            case ControlPanel::PANEL_API_VERSION_TOO_HIGH:
                return (new DAViewModel\SharedCodeUpdateRequiredViewModel());

            case ControlPanel::PANEL_API_VERSION_TOO_LOW:
            case ControlPanel::PANEL_API_VERSION_UNKNOWN:
                return (new DAViewModel\PluginUpdateRequiredViewModel());

            //no default
        }

        switch ($do) {

            case 'lscwp_manage':
                $viewModel = $this->lscwpManage($mgrStep);
                break;

            case 'lscwp_mass_enable_disable':
                $viewModel = $this->lscwpMassEnableDisableControl($step);
                break;

            case 'lscwpVersionManager':
                $viewModel = $this->lscwpVersionManager($step);
                break;

            case 'dash_notifier':
                $viewModel = $this->dashNotifier($dashStep);
                break;
        }

        return $viewModel;
    }

    /**
     *
     * @since 2.0
     *
     * @return object  ViewModel object.
     */
    protected function getMainMenuViewModel()
    {
        return (new DAViewModel\Open_MainMenuViewModel());
    }

    /**
     * Note: Assume that cache roots are set for OLS in DA env.
     *
     * @since 2.0
     *
     * @param int  $mgrStep
     * @return object  ViewModel object.
     */
    protected function lscwpManage( $mgrStep )
    {
        $panelEnv = $this->getControlPanelInstance();

        $lscmDataFiles = Context::getLSCMDataFiles();

        $model = new DAPanelWPInstallStorage($lscmDataFiles['dataFile'],
                $lscmDataFiles['custDataFile']);

        $panelController =
                new DAPanelController($panelEnv, $model, $mgrStep);

        /**
         * Pre-initialize DAPanelPluginVersion instance for viewModel.
         */
        PluginVersion::getInstance();

        $newMgrStep = $panelController->manageCacheOperations();

        switch ($newMgrStep) {
            case DAPanelController::MGR_STEP_SCAN:
            case DAPanelController::MGR_STEP_DISCOVER_NEW:
                return (new LscViewModel\ScanProgressViewModel($newMgrStep));
            case DAPanelController::MGR_STEP_REFRESH_STATUS:
                return (new LscViewModel\RefreshStatusProgressViewModel());
            case DAPanelController::MGR_STEP_MASS_UNFLAG:
                return (new LscViewModel\UnflagAllProgressViewModel());
            default:
                return (new LscViewModel\ManageViewModel($model));
        }
    }

    /**
     * Note: Assume that cache roots are set for OLS in DA env.
     *
     * @since 2.0
     *
     * @param int  $step
     * @return object  ViewModel object.
     */
    protected function lscwpMassEnableDisableControl( $step )
    {
        $lscmDataFiles = Context::getLSCMDataFiles();
        $model = new DAPanelWPInstallStorage($lscmDataFiles['dataFile'],
                $lscmDataFiles['custDataFile']);

        if ( $step == 1 || $step == 2 ) {
            $allowedActions = array( 'enable', 'disable' );

            $action = DALiteSpeedUtil::get_request_var('act');

            if ( in_array($action, $allowedActions) ) {
                return $this->lscwpMassEnableDisable($action, $step, $model);
            }
        }
        elseif ( $step == 0 && $model->getError() != 0 ) {
            return (new LscViewModel\DataFileMsgViewModel($model));
        }

        return (new LscViewModel\MassEnableDisableViewModel($model));
    }

}
