<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel\Extend;

use \Lsc\Wp\LSCMException;
use \Lsc\Wp\Panel\DirectAdmin;
use \LsPanel\DAEscalate;
use \LsPanel\DALiteSpeedUtil;

/**
 * @since 2.0
 */
class DirectAdminPanel extends DirectAdmin
{

    /**
     *
     * @since 2.0
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    protected function serverCacheRootSearch()
    {
        $args = array('filePath' => $this->apacheConf);
        $apacehConfExists = DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_FILE_EXISTS, $args);

        if ( $apacehConfExists ) {
            return $this->getCacheRootSetting($this->apacheConf);
        }

        return '';
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $svrCacheRoot
     */
    public function setServerCacheRoot( $svrCacheRoot = '' )
    {
        DAEscalate::escalate(true, DAEscalate::MGR_CMD_SET_SERVER_CACHE_ROOT);

        $this->serverCacheRoot = $this->defaultSvrCacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $vhConf
     * @param string  $vhCacheRoot
     */
    public function writeVHCacheRoot( $vhConf, $vhCacheRoot = 'lscache' )
    {
        $args = array ( 'vhConf' => $vhConf );
        DAEscalate::escalate(true, DAEscalate::MGR_CMD_WRITE_VH_CACHE_ROOT,
                $args);
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $confDir  Directory to be searched.
     * @return string
     */
    public function daVhCacheRootSearch( $confDir )
    {
        $args = array( 'confDir' => $confDir );
        $cacheRoot = DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_DA_VH_CACHE_ROOT_SEARCH, $args);

        return $cacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $file
     * @return string
     */
    public function getCacheRootSetting( $file )
    {
        $args = array( 'confFile' => $file );
        $cacheRoot = DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_GET_CACHE_ROOT_SETTING, $args);

        return $cacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    public function getLSWSCacheRootSetting()
    {
        $svrcacheRoot = DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_GET_LSWS_CACHE_ROOT_SETTING);

        return $svrcacheRoot;
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $vhConf
     * @param string  $vhCacheRoot
     */
    public function createVHConfAndSetCacheRoot( $vhConf,
            $vhCacheRoot = 'lscache' )
    {
        $args = array ( 'vhConf' => $vhConf );
        DAEscalate::escalate(true,
                DAEscalate::MGR_CMD_CREATE_VHCONF_AND_SET_CACHE_ROOT, $args);
    }

    /**
     *
     * @since 2.0
     */
    public function applyVHConfChanges()
    {
        DAEscalate::escalate(true, DAEscalate::MGR_CMD_APPLY_VHCONF_CHANGES);
    }

    /**
     *
     * @since 2.0
     */
    protected function prepareDocrootMap()
    {
        $this->docRootMap =
                DAEscalate::escalate(true, DAEscalate::MGR_CMD_GET_DOCROOT_MAP);
    }

    /**
     *
     * @since 2.0
     *
     * @throws LSCMException
     */
    public function verifyCacheSetup()
    {
        if ( !$this->isCacheEnabled() ) {
            $msg = 'LSCache is not included in the current LiteSpeed license. Please purchase the '
                    . 'LSCache add-on or upgrade to a license type that includes LSCache and try '
                    . 'again.';

            throw new LSCMException($msg, LSCMException::E_PERMISSION);
        }

        $restartRequired = false;

        if ( self::NOT_SET == $this->getServerCacheRoot() ) {
            $this->setServerCacheRoot();
            $restartRequired = true;
        }

        if ( self::NOT_SET == $this->getVHCacheRoot() ) {
            $this->setVHCacheRoot();
            $restartRequired = true;
        }

        if ( $restartRequired ) {
            $output = array();
            DALiteSpeedUtil::restartLSWS($output);
        }
    }

    /**
     *
     * @since 2.0
     *
     * @return boolean
     */
    public function isCacheEnabled()
    {
        return DAEscalate::escalate(true, DAEscalate::MGR_CMD_IS_CACHE_ENABLED);
    }

}
