<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel\View\Model;

use \Lsc\Wp\LSCMException;
use \LsPanel\DALiteSpeedControllerBase;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\DAPluginException;
use \LsPanel\DAPluginLogger;
use \LsPanel\Extend\DAPanelPluginVersion;
use \LsPanel\View\Model\BaseViewModel;

/**
 * @since 2.0
 */
class MainMenuViewModel extends BaseViewModel
{

    /**
     * @since 2.0
     * @var string
     */
    const FLD_AP_PID = 'apPid';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_LS_PID = 'lsPid';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_MAIN_MENU_INFO = 'mainMenuInfo';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_PLUGIN_VER = 'pluginVer';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_CURR_LSCWP_VER = 'currLscwpVer';

    /**
     * @since 2.0
     * @var string
     */
    const FLD_LATEST_LSCWP_VER = 'newLscwpVer';

    /**
     *
     * @since 2.0
     */
    public function __construct()
    {
        parent::__construct();

        $this->init();
    }

    /**
     *
     * @since 2.0
     */
    protected function init( )
    {
        $this->setPidData();
        $this->setMainMenuInfoData();
        $this->setDAPluginVerData();
        $this->setLscwpVerData();
    }

    /**
     *
     * @since 2.0
     *
     * @throws DAPluginException
     */
    protected function setPidData()
    {
        $lsPid = DALiteSpeedUtil::getLSPID();

        if ( $lsPid > 0 && !defined('LSWS_HOME') ) {
            throw new DAPluginException('lsPid detected with LSWS_HOME undefined');
        }

        $this->tplData[self::FLD_AP_PID] = DALiteSpeedUtil::getApachePID();
        $this->tplData[self::FLD_LS_PID] = $lsPid;
    }

    /**
     *
     * @since 2.0
     */
    protected function setMainMenuInfoData()
    {
        $this->tplData[self::FLD_MAIN_MENU_INFO] =
                DALiteSpeedUtil::getMainMenuInfo();
    }

    /**
     *
     * @since 2.0
     */
    protected function setDAPluginVerData()
    {
        $this->tplData[self::FLD_PLUGIN_VER] =
                DALiteSpeedControllerBase::MODULE_VERSION;
    }

    /**
     *
     * @since 2.0
     */
    protected function setLscwpVerData()
    {
        $currLscwpVer = $latestLscwpVer = '';

        if ( SHARED_CODE_LOADED ) {
            $verMgr = DAPanelPluginVersion::getInstance();

            try {
                $currLscwpVer = $verMgr->getCurrentVersion();
            }
            catch ( LSCMException $e ) {
                $msg = $e->getMessage() . ' Could not get active LSCWP version.';
                DAPluginLogger::debug($msg);
            }

            try {
                $latestLscwpVer = $verMgr->getLatestVersion();
            }
            catch ( LSCMException $e ) {
                $msg = $e->getMessage() . ' Could not get latest LSCWP version.';
                DAPluginLogger::debug($msg);
            }
        }

        $this->tplData[self::FLD_CURR_LSCWP_VER] = $currLscwpVer;
        $this->tplData[self::FLD_LATEST_LSCWP_VER] = $latestLscwpVer;
    }

    /**
     *
     * @since 2.0
     *
     * @return string
     */
    public function getTpl()
    {
        return realpath(__DIR__ . '/../Tpl') . '/MainMenu.tpl';
    }

}
