<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel\View;

use \Lsc\Wp\Context\Context;
use \Lsc\Wp\LSCMException;
use \LsPanel\DALiteSpeedUtil;
use \LsPanel\DAPluginException;
use \LsPanel\DAPluginLogger;

/**
 * @since 2.0
 */
class View
{

    /**
     * @since 2.0
     * @var object
     */
    private $viewModel;

    /**
     * @since 2.0
     * @var string
     */
    private $panelViewDir = __DIR__;

    /**
     *
     * @since 2.0
     *
     * @param object  $viewModel
     */
    public function __construct( $viewModel )
    {
        $this->viewModel = $viewModel;
    }

    /**
     *
     * @since 2.0
     */
    public function display()
    {
        $this->loadTpl($this->viewModel->getTpl());
    }

    /**
     *
     * @since 2.0
     *
     * @param string  $tplPath
     * @throws DAPluginException
     */
    private function loadTpl( $tplPath )
    {
        $tplFile = basename($tplPath);
        $custTpl = "{$this->panelViewDir}/Tpl/Cust/{$tplFile}";

        if ( file_exists($custTpl) ) {
            $activeTpl = $custTpl;
        }
        elseif ( file_exists($tplPath) ) {
            $activeTpl = $tplPath;
        }
        else {
            throw new DAPluginException("Could not load page template {$tplPath}.");
        }

        $do = DALiteSpeedUtil::get_request_var('do');

        $d = array(
            'do' => $do
        );
        $this->loadTplBlock('PageHeader.tpl', $d);

        include $activeTpl;

        $this->loadTplBlock('PageFooter.tpl', $d);
    }

    /**
     * Used by the page template to load sub-template blocks.
     *
     * @since 2.0
     *
     * @param string  $tplName
     * @param array   $d        Sub-template data.
     * @param bool    $shared   True if block tpl is found in shared directory.
     * @return null
     * @throws DAPluginException
     */
    private function loadTplBlock( $tplName, $d, $shared = false )
    {
        $custTpl = "{$this->panelViewDir}/Tpl/Cust/Blocks/{$tplName}";

        if ( file_exists($custTpl) ) {
            include $custTpl;
            return;
        }

        if ( $shared
                && include_once LSWS_HOME . '/add-ons/webcachemgr/src/Context/Context.php' ) {

            try {
                $sharedTplDir = Context::getOption()->getSharedTplDir();
                $tplPath = "{$sharedTplDir}/Blocks/{$tplName}";
            }
            catch ( LSCMException $e ) {
                $msg = $e->getMessage() . ' Unable to get shared template directory.';
                DAPluginLogger::error($msg);

                throw new DAPluginException($msg);
            }
        }
        else {
            $tplPath = $this->panelViewDir . "/Tpl/Blocks/{$tplName}";
        }

        if ( file_exists($tplPath) ) {
            include $tplPath;
        }
        else {
            throw new DAPluginException("Could not load block template {$tplPath}.");
        }
    }

}
