<?php

/* * ******************************************
 * LiteSpeed Web Server Plugin for DirectAdmin
 *
 * @author LiteSpeed Technologies, Inc. (https://www.litespeedtech.com)
 * @copyright (c) 2019
 * @since 2.0
 * ******************************************* */

namespace LsPanel;

use \Lsc\Wp\LSCMException;
use \LsPanel\DALiteSpeedController;
use \LsPanel\DAOpenLiteSpeedController;
use \LsPanel\DAPluginException;
use \LsPanel\DAPluginLogger;

require_once __DIR__ . '/autoloader.php';

/**
 *
 * @since 2.0
 *
 * @param string  $msg
 */
function displayCustomExceptionMsg( $msg )
{
    if ( defined('LSWS_AJAX_CALL') && LSWS_AJAX_CALL != false ) {
        echo "HTTP/1.1 200 OK\nContent-Type: text/html\n\n";
    }

    DAPluginLogger::error($msg);
    echo "<h1>Exception Caught - {$msg}</h1>";
}

try
{
    DAPluginLogger::Initialize();
    DAPluginLogger::setAdditionalTagInfo(
            "[{$_SERVER['REMOTE_ADDR']}-" . getmypid() . ']');
    DAPluginLogger::preCreateLogFile();

    /**
     * Set PHP session ID based on DirectAdmin session ID. Because the plugin
     * is run using a CLI PHP binary, PHP session ID must be set manually.
     */
    session_id(md5($_SERVER['SESSION_ID']));

    if ( basename(realpath('/usr/local/lsws/bin/lshttpd')) == 'openlitespeed' ) {
        define('LSWS_SERVER_TYPE', 'open');
        (new DAOpenLiteSpeedController())->run();
    }
    else {
        define('LSWS_SERVER_TYPE', 'ent');
        (new DALiteSpeedController())->run();
    }
}
catch ( DAPluginException $e )
{
    displayCustomExceptionMsg($e->getMessage());
}
catch ( LSCMException $e )
{
    displayCustomExceptionMsg($e->getMessage());
}
